function [deltaE,Icor,Irep,c,slope,infpoint]=pHfit(Iexp,esv,E,isshift,pH);
%
%--------------------------------------------------------------------------
% pHfit, by J.M. Daz, ; March 2009, Last revised April 2010
%-------------------------------------------------------------------------- 
%
% corrects the matrix for the potential shift of one or more components
% (deltaE) in pH titrations
%
% Iexp is the experimental matrix, where potentials must be in columns and pH in rows
% esv (in column-wise form) is the estimation of the unit voltammograms at the reference potentials
% Icor, Irep are the corrected and reproduced matrices, respectively
% c contains the concentration profiles (in arbitrary units)
% E is the potential vector (in a column)
% pH is the pH value 
% slope (optional) and infpoint (optional) are the maximum slope (in abs. values) and the 
% inflexion point of the deltaE vs. pH plot
%
% isshift is a vector in the form [1,0,0,2,....] which denotes with '0' the components/signals 
% that do not move along the potential axis, with '1' these which move in
% an unpredicted way, with '2' these moving linearly, with '3' these moving according to a sigmoidal
% evolution with pH and with '4' these according to a sigmoidal evolution
% which starts or finishes just at the reference potential defined in esv
% (sigmoid with zero baseline)
%
% 4 additional rows can be added to isshift to denote: 
% i) and ii), the number of experiment where every component appears and disappears (existence range). 
%             (Out of this range the concentration is fixed to zero during the whole fitting);
% iii) and iv), the numbers of experiment which define the fitting range for every component in the
%              case of initial estimation of sigmoidal movement with pH 
%             (The existence range is equal or wider than the fitting range):
% 
% syntax:  [deltaE,Icor,Irep,c,slope,infpoint]=pHfit(Iexp,esv,E,isshift,pH);

warning('off','MATLAB:dispatcher:InexactMatch');subplot(1,1,1);format short;

%checking dimensions
[nexp,npot]=size(Iexp);
[npot2,ncomp]=size(esv);
[npot3,nothing]=size(E);
if nexp>npot,
	disp('Warning: number of potentials is lower than number of experiments');
	disp('Check data matrix');
end
if npot2~=npot, error('Iexp and esv dimensions must agree');end
if npot3~=npot, error('Iexp and E dimensions must agree');end
if nargin==5,[npH,nothing]=size(pH);
	if npH~=nexp, error('Iexp and pH dimensions must agree');end
else npH=nexp;	
end

%setting up the existence and fitting ranges
if nargin==5
	exist1=isshift(2,:);
	exist2=isshift(3,:);
	fit1=isshift(4,:);
	fit2=isshift(5,:);
elseif nargin==3
	exist1=isshift(2,:);
	exist2=isshift(3,:);
	fit1=exist1;
	fit2=exist2;
elseif nargin==1
	exist1=ones(1,ncomp);
	exist2=nexp*ones(1,ncomp);
	fit1=exist1;
	fit2=exist2;
else error('Number of rows of isshift must be 1,3 or 5');
end
	
%normalized variable number of column
for i=1:npot,x(i,1)=i/npot;end

%normalization of Iexp and esv (Iexpn and v)
maxI=max(max(Iexp));
Iexpn=Iexp./maxI;
maxv=max(max(esv));
v=esv./maxv;
v0=v;%save initial estimation of pure voltammograms as v0 (in prevention of further baseline changes)

% initialization of delta0
deltaE0=zeros(nexp,ncomp);

% estimation of deltaE0 (in potential values)

%generation of different filters to be used in Savitzky-Golay 1st and 2nd derivative
fil631=cgolay(6,3,1);fil632=cgolay(6,3,2);
fil321=cgolay(3,2,1);fil322=cgolay(3,2,2);
	
%selecting the range of existence of each peak
for k=1:ncomp;
	plot(Iexp(exist1(1,k):exist2(1,k),:)','g');hold;plot(esv,'k');plot(esv(:,k),'r.')
	if isshift(1,k)~=0,
		title('select with mouse the potential range of peak maximum'),
		[x1(k,1),nothing]=ginput(1);[x2(k,1),nothing]=ginput(1);
		iin(k,1)=round(min(x1(k,1),x2(k,1)));ifin(k,1)=round(max(x1(k,1),x2(k,1)));
		hold;
	else title('peak will not move, press enter');pause;hold
	end
end
%initial estimation of deltaE (deltaE0) inside the potential and concentration existing range
for i=1:nexp; 
	for k=1:ncomp
		if isshift(1,k)~=0 && i >= exist1(1,k) && i <= exist2(1,k),
				%maximum of the pure voltammogram
		     [nothing,iesmax]=max(v0(:,k));E0max=E(iesmax,1);
           %selecting the potential range and other related ranges			
				E2=E(iin(k,1):ifin(k,1),1); 
				I=(Iexpn(i,iin(k,1):ifin(k,1)));
            %looking for maxima in the experimental matrix
            %width of the ranges
            [nrI,ncI]=size(I);
				%estimation of second derivative
            if ncI>20, dI2=(golay(fil632,I'))';
            else dI2=(golay(fil322,I'))';
				end
            %range where 2nd derivative is negative
            [nothing,idI2neg]=find(dI2<0);
            %looking for the higher value inside this range
            [nothing,imax]=max(I(idI2neg));E0=E2(idI2neg(imax));
            %computing deltaE0 (using the previous value if no relative maxima are found)
				if isempty(idI2neg)==0,deltaE0(i,k)=E0-E0max;
				elseif i>1, deltaE0(i,k)=deltaE0((i-1),k);
				else deltaE0(i,k)=0;
				end
            %discarding if the peak is too small
            %if I(idI2neg(imax))<0.01, deltaE0(i,k)=0; end
		else deltaE0(i,k)=0;  
		end  
	end
end
deltaE0old=deltaE0; %saves for final plot

%making deltaE0u (column vector) for unconstrained shift
deltaE0u=[];
for k=1:ncomp,
	if isshift(1,k)==1,
	deltaE0u=[deltaE0u;deltaE0(:,k)];
	end
end
if isempty(deltaE0u)==0;[nrdeltaEu,nothing]=size(deltaE0u);end
	
% initial estimation of linear and sigmoidal parameters

a0=zeros(ncomp,3);%initialization of a0
for k=1:ncomp,
	deltaE0r=[];
	%restriction of the experiments where every signal can be fitted
		pHr=pH(fit1(1,k):fit2(1,k),1);
		deltaE0r=deltaE0(fit1(1,k):fit2(1,k),k);
		[nrr,nothing]=size(deltaE0r);
		if fit1(1,k) > 1, deltaE0old(1:fit1(1,k),k)=zeros; end
		if fit2(1,k) < nexp, deltaE0old(fit2(1,k):nexp,k)=zeros;end
		deltaE0b(:,k)=deltaE0old(:,k);%b indicates relative to baseline-subtracted sigmoid,
		%but defined for the whole pH range
	if isshift(1,k)==2,%estimating linear parameters in the form y = a1+a2*x (a3=0);
		P=polyfit(pHr,deltaE0r,1);
		a1=P(1,2);a2=P(1,1);
		a0(k,:)=[a1,a2,0];
		deltaE0(:,k)=a1+a2.*pH;
	end	
	if isshift(1,k)>=3,
		%baseline subtraction (movement of reference pure voltammogram)
		if isshift(1,k)==3,
			ddeltaE=golay(fil321,deltaE0r);
			[iEb,nothing]=find(abs(ddeltaE<0.01)&deltaE0r~=0);
			if isempty(iEb)==0, 
				Ebase(1,k)=min(deltaE0r(iEb));
			elseif min(deltaE0r)~=0,
				Ebase(1,k)=min(deltaE0r);
			else Ebase(1,k)=0;
			end
			Evbase=E-Ebase(1,k);
			v(:,k)=negtozero(spline(E,v0(:,k),Evbase));
			deltaE0r=deltaE0r-Ebase(1,k);
			deltaE0b(fit1(1,k):fit2(1,k),k)=deltaE0r;
		elseif isshift(1,k)==4, Ebase(1,k)=0;
		end
		if sum(deltaE0r)<0, sign=-1; else sign=1;end
		a1=sign*max(abs(deltaE0r));% computing a1 from wave height
      dif=abs(deltaE0r-a1/2);
		% computing a3 as minimum distance to half-wave position
      [nothing,idif]=min(dif);
		a3=pHr(idif);
		if idif+1<=nrr && idif-1>0,
		slope0(k,1)=(deltaE0r(idif+1,1)-deltaE0r(idif-1,1))/(pHr(idif+1,1)-pHr(idif-1,1));
		elseif idif+1<=nrr, slope0(k,1)=(deltaE0r(idif+1,1)-deltaE0r(idif,1))/(pHr(idif+1,1)-pHr(idif,1)); 
		elseif idif-1>0, slope0(k,1)=(deltaE0r(idif,1)-deltaE0r(idif-1,1))/(pHr(idif,1)-pHr(idif-1,1));
		else slope0(k,1)=0.05;
		end
		a2=-slope0(k,1)*4/a1;
		a0(k,:)=[a1,a2,a3];
		deltaE0(:,k)=a1./(1+exp(a2.*(pH-a3)));
	end	
end

%unfold a0 as a vector to be able to compute Jacobian (a3 is not considered in linear fit)

a0unfold=[];
for k=1:ncomp,
	if isshift(1,k)>=3,
	a0unfold=[a0unfold;(a0(k,:))'];
	elseif isshift(1,k)==2
	a0unfold=[a0unfold;(a0(k,1:2))'];	
	end %immobile or unconstrained components are not included in a0
end
a0=a0unfold;
[nra,nothing]=size(a0);%computing the number of rows of a0

%estimation of c

c=[];kexist=zeros(nexp,ncomp);
%initialization of c and further construction only with the components existing in every experiment
for i=1:nexp
    vshift=[];%reset unit voltammograms at each new experiment
 	for k=1:ncomp, 
        if i < exist1(1,k) || i > exist2(1,k); 
            c(i,k)=0; kexist(i,k)=0; 
        elseif deltaE0(i,k)~=0, 
            Eshift=E-deltaE0(i,k);
            vshift=[vshift,negtozero(spline(E,v(:,k),Eshift))];
				kexist(i,k)=1;
		  elseif deltaE0(i,k)==0,
				vshift=[vshift,v(:,k)];
				kexist(i,k)=1;
		  end
	end
	cnonzero=negtozero(Iexpn(i,:)/vshift');
	for k=1:ncomp
		if kexist(i,k)==0, 
			c(i,k)=0; 
		elseif kexist(i,k)==1, 
			[nothing,nck]=size(cnonzero);
			c(i,k)=cnonzero(1,1);
			if nck>1,cnonzero=cnonzero(1,2:nck);end
		end
    end	
    for k=1:ncomp,%forcing again zero concentration in the regions were the peak does not exist
		if i < exist1(1,k) || i > exist2(1,k); c(i,k)=0; end 
	end
end
%plot of the estimated concentration
plot(pH,c,'-o');xlabel('pH');ylabel('c');title('initial estimation of c: press any key');pause

%plot initial estimation

plot(pH,deltaE0b,'o');xlabel('pH');ylabel('deltaE0 / V');%deltaE0b: raw, baseline-corrected estimates
title('initial estimation of deltaE: press any key');
hold;plot(pH,deltaE0,'-');pause;hold;%deltaE0: including initial sigmoidal estimates 
	
	
%fitting %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%fitting of sigmoidal parameters with the rest of initial deltaE0 values fixed
options=optimset('Display','off','Diagnostics','off','MaxIter',20,'TolX',0.001,'TolFun',0.001);
cv=[nexp,isshift(1,:);pH,c;pH,deltaE0;E,v;0,exist1(1,:);0,exist2(1,:)];
lb=ones(nra,1)*(-15);ub=ones(nra,1)*15;
[a,resnorm,residual,exitflag,output,lambda,jacobian]=lsqcurvefit('shiftcalc31',a0,cv,Iexpn,lb,ub,options);

%folding of sigmoidal parameters fitted (including linear parameters or zero for the non-sigmoidal)
aunfold=a;afold=[];
for k=1:ncomp;
	if isshift(1,k)>=3,
		[nra,nothing]=size(aunfold);
		afold=[afold,aunfold(1:3,1)];
		if nra>4,aunfold=aunfold(4:nra,1);end
	elseif isshift(1,k)==2,
		[nra,nothing]=size(aunfold);
		afold=[afold,[aunfold(1:2,1);0]];
		if nra>3,aunfold=aunfold(3:nra,1);end
	else afold(:,k)=zeros(3,1);
	end
end
[nothing,ncafold]=size(afold);

%fitting of unconstrained deltaE with the (previously optimized) linear or sigmoidal
%potential shifts fixed
if isempty(deltaE0u)==0;
	disp(' ');disp('Sigmoidal fitting done.Proceed with unconstrained fitting');disp(' ');
	options=optimset('Display','off','Diagnostics','off','MaxIter',20,'TolX',0.001,'TolFun',0.001);
	cv=[];cv=[nexp,isshift(1,:);pH,c;E,v;0,exist1(1,:);0,exist2(1,:);zeros(3,1),afold];
	lb=ones(nrdeltaEu,1)*(-2);ub=ones(nrdeltaEu,1)*2;
	[deltaEu]=lsqcurvefit('shiftcalc32',deltaE0u,cv,Iexpn,lb,ub,options);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%folding optimized a parameters
%computind std. dev. of adjusted a parameters from Jacobian
%(in case of sigmoidal pH fitting)
[nra,nothing]=size(a);
P=inv(jacobian'*jacobian);%variance-covariance matrix
% standard deviation of every fitted parameter
for j=1:nra,
	stda(j,1)=sqrt(resnorm/(nexp*npot-nra)*P(j,j));
end

%compute optimized deltaE values (recovering baseline)
aunfold=a;stdaunfold=stda;
for k=1:ncomp,
	[nra,nothing]=size(aunfold);
	if isshift(1,k)>=3,%sigmoidal evolution
		a1(k,1)=aunfold(1,1);a2(k,1)=aunfold(2,1);a3(k,1)=aunfold(3,1);
		stda1(k,1)=stdaunfold(1,1);stda2(k,1)=stdaunfold(2,1);stda3(k,1)=stdaunfold(3,1);
		if nra>4,aunfold=aunfold(4:nra,1);stdaunfold=stdaunfold(4:nra,1);end
        deltaE(:,k)=Ebase(1,k)+a1(k,1)./(1+exp(a2(k,1).*(pH-a3(k,1)))); 
	elseif isshift(1,k)==2,% linear evolution	  
		a1(k,1)=aunfold(1,1);a2(k,1)=aunfold(2,1);
		stda1(k,1)=stdaunfold(1,1);stda2(k,1)=stdaunfold(2,1);
		if nra>3,aunfold=aunfold(3:nra,1);stdaunfold=stdaunfold(3:nra,1);end
        deltaE(:,k)=a1(k,1)+a2(k,1).*pH;   
	elseif isshift(1,k)==1,% unconstrained evolution
		a1(k,1)=0;a2(k,1)=0;a3(k,1)=0;
		stda1(k,1)=0;stda2(k,1)=0;stda3(k,1)=0;
		[nrE,nothing]=size(deltaEu);
        deltaE(:,k)=deltaEu(1:npH,1);
		if npH+1>nrE,deltaEu=deltaEu(npH+1:nrE,1);end
    elseif isshift(1,k)==0,% immobile component
      a1(k,1)=0;a2(k,1)=0;a3(k,1)=0;
		stda1(k,1)=0;stda2(k,1)=0;stda3(k,1)=0;
		deltaE(:,k)=zeros(npH,1); 
	end	
end

% display of maximum slope and inflexion point in the case of sigmoidal pH variation
%computing std. dev. in slope (error propagation through relative std.dev.)
slope=zeros(ncomp,1);stdslope=zeros(ncomp,1);infpoint=zeros(ncomp,1);stdinfpoint=zeros(ncomp,1);
for k=1:ncomp,
	if isshift(1,k)>=3,
		slope(k,1)=abs(a1(k,1).*a2(k,1)/4);infpoint(k,1)=a3(k,1);
		stdslope(k,1)=0.25*slope(k,1).*sqrt((stda1(k,1)./a1(k,1)).^2+(stda2(k,1)./a2(k,1)).^2);
		stdinfpoint(k,1)=stda3(k,1);
	elseif isshift(1,k)==2,
		slope(k,1)=abs(a2(k,1));infpoint(k,1)=0;
		stdslope(k,1)=stda2(k,1);
		stdinfpoint(k,1)=0;
	end	
end

disp('maximum slope (in absolute value):');slope=[slope,stdslope]
disp('inflexion point');infpoint=[infpoint,stdinfpoint]

%plotting deltaE and c

plot(pH,deltaE0old,'o');xlabel('pH');ylabel('deltaE / V');
title('fitted values of deltaE and c: press any key');
hold;%plotting only in the existence range
for k=1:ncomp, 
	if isshift(1,k)>1,plot(pH(exist1(1,k):exist2(1,k),1),deltaE0(exist1(1,k):exist2(1,k),k),'k:'),end
end;
plot(pH,deltaE,'-');
pause;hold;

%corrected (Icor) and reproduced (Irep) matrices, 
%error matrix (Ierr)
%lack of fit between Irep and Iexp

ss1=0;ss2=0;cnonzero=[];
for i=1:nexp
	vshift=[];Eshift=[];
    for k=1:ncomp, 
		if kexist(i,k)~=0 && deltaE(i,k)~=0,
            Eshift=E-deltaE(i,k);
            vshift=[vshift,negtozero(spline(E,v0(:,k),Eshift))];
        elseif kexist(i,k)~=0 && deltaE(i,k)==0,
            vshift=[vshift,v0(:,k)];
        end
    end
   cnonzero=negtozero(Iexpn(i,:)/vshift');
   Irep(i,:)=negtozero(maxI*cnonzero*vshift');
   for k=1:ncomp
		if kexist(i,k)==0, 
			c(i,k)=0; 
        else [nothing,nck]=size(cnonzero);
			c(i,k)=cnonzero(1,1);
			if nck>1,cnonzero=cnonzero(1,2:nck);end
		end
   end	
   Icor(i,:)=maxI*c(i,:)*v0';
   for j=1:npot
   ss1=ss1+(Iexp(i,j)-Irep(i,j))^2;
   ss2=ss2+(Iexp(i,j))^2;
   end
end

Ierr=Irep-Iexp;
lof=((ss1/ss2)^0.5)*100;
disp(' ');disp('lack of fit (%) :  ');disp(lof)

%plotting optimized concentration

figure
plot(pH,c,'o-');title('fitted concentration / a.u.: press any key');xlabel('pH');ylabel('c / a.u.');pause;

%plotting final results
figure
subplot(2,2,1);plot(-E,Iexp);title('experimental matrix');ylabel('current / A');
axis([min(-E) max(-E) 0 1.1*maxI]),
subplot(2,2,3);plot(-E,Irep);title('reproduced matrix');ylabel('current / A');
axis([min(-E) max(-E) 0 1.1*maxI]),
subplot(2,2,2);plot(-E,Ierr);title('error matrix');xlabel('-E / V');ylabel('current / A');
axis([min(-E) max(-E) -1.1*maxI/2 1.1*maxI/2]),
subplot(2,2,4);plot(-E,Icor);title('corrected matrix');xlabel('-E / V');ylabel('current / A');
axis([min(-E) max(-E) 0 1.1*maxI]),


